
#include <Arduino.h>
#include "PinChangeInt.h"
#include "Pins.h"
#include "mode.h"
#include "Command.h"
#include "Rgb.h"
#include "Ultrasonic.h"
#include "voltage.h"
#include "MsTimer2.h"
#include "KalmanFilter.h"
#include "I2Cdev.h"
#include "MPU6050.h"
#include "Wire.h"
MPU6050 mpu;
KalmanFilter kalmanfilter;

//Setting PID parameters
// Balance uses proportional and derivative but no integral
double kp_balance = 55, kd_balance = 0.75;

// Speed uses proportional and integral but no derivative
double kp_speed = 10, ki_speed = 0.26;


double kp_turn = 2.5, kd_turn = 0.5;

//Setting MPU6050 calibration parameters
double angle_zero = 0;            //x axle angle calibration
double angular_velocity_zero = 0; //x axle angular velocity calibration

volatile unsigned long encoder_count_right_a = 0;
volatile unsigned long encoder_count_left_a = 0;
int16_t ax, ay, az, gx, gy, gz;
float dt = 0.005, Q_angle = 0.001, Q_gyro = 0.005, R_angle = 0.5, C_0 = 1, K1 = 0.05;

int encoder_left_pulse_num_speed = 0;
int encoder_right_pulse_num_speed = 0;
double speed_control_output = 0;
double rotation_control_output = 0;
double speed_filter = 0;
int speed_control_period_count = 0;
double car_speed_integeral = 0;
double speed_filter_old = 0;
int setting_car_speed = 0;
int setting_turn_speed = 0;
double pwm_left = 0;
double pwm_right = 0;
float kalmanfilter_angle;
// char balance_angle_min = -27;
// char balance_angle_max = 27;
char balance_angle_min = -22;
char balance_angle_max = 22;

unsigned long start_prev_time = 0;
boolean carInitialize_en = true;
  static unsigned long start_time = 0;
  
void setup() {

  Serial.begin(115200);
  carInitialize();
  carStop();
 // carForward(0);
  motion_mode = FORWARD;  
  setting_car_speed = 10;
}
void loop() {
 // Serial.println("Motion mode "+motion_mode);
 //motion_mode = START;
  if (millis() - start_time == 2000) {
    if(kalmanfilter_angle >= balance_angle_min){
      motion_mode = START;
      Serial.println("Motion mode set to start");
    }
      
  }
  if (millis() - start_time < 10) {
    function_mode = IDLE;
    motion_mode = STOP;
    carStop();
  }  
}

void loop1() {

  motion_mode = STOP;
  
}

void carStop() {
  digitalWrite(AIN1, HIGH);
  digitalWrite(BIN1, LOW);
  digitalWrite(STBY_PIN, HIGH);
  analogWrite(PWMA_LEFT, 0);
  analogWrite(PWMB_RIGHT, 0);
}

void carForward(unsigned char speed) {
  digitalWrite(AIN1, 0);
  digitalWrite(BIN1, 0);
  analogWrite(PWMA_LEFT, speed);
  analogWrite(PWMB_RIGHT, speed);
}

void carBack(unsigned char speed) {
  digitalWrite(AIN1, 1);
  digitalWrite(BIN1, 1);
  analogWrite(PWMA_LEFT, speed);
  analogWrite(PWMB_RIGHT, speed);
}

// This function is called in the background using a software timer (in carInitialize() below)
void balanceCar() {

  sei();
  //Serial.println("Balance");
  // Get the encoder pulses
  encoder_left_pulse_num_speed += pwm_left < 0 ? -encoder_count_left_a : encoder_count_left_a;
  encoder_right_pulse_num_speed += pwm_right < 0 ? -encoder_count_right_a : encoder_count_right_a;
  encoder_count_left_a = 0;
  encoder_count_right_a = 0;
  
  // Get the accelerometer values
  mpu.getMotion6(&ax, &ay, &az, &gx, &gy, &gz);
  // Filter to remove noise
  kalmanfilter.Angle(ax, ay, az, gx, gy, gz, dt, Q_angle, Q_gyro, R_angle, C_0, K1);
  // get the angle
  kalmanfilter_angle = kalmanfilter.angle;
  // get the balance control using a P-D controller (no I)
  double balance_control_output = kp_balance * (kalmanfilter_angle - angle_zero) + kd_balance * (kalmanfilter.Gyro_x - angular_velocity_zero);

  Serial.print("Mode = ");Serial.print(motion_mode); Serial.print(" ");
  Serial.print("Angle ");Serial.print(kalmanfilter_angle);Serial.print(" ");
  
 // Serial.println(balance_control_output);
    
  speed_control_period_count++;
  if (speed_control_period_count >= 8) {
    speed_control_period_count = 0;
	// Calculate speed from the encoder pulses
    double car_speed = (encoder_left_pulse_num_speed + encoder_right_pulse_num_speed) * 0.5;
    encoder_left_pulse_num_speed = 0;
    encoder_right_pulse_num_speed = 0;
	// Averaging filter
    speed_filter = speed_filter_old * 0.7 + car_speed * 0.3;
    speed_filter_old = speed_filter;
	// calculate integral value ready for P-I controller
    car_speed_integeral += speed_filter;
    car_speed_integeral += -setting_car_speed;
    car_speed_integeral = constrain(car_speed_integeral, -3000, 3000);
	// P-I controller
    speed_control_output = -kp_speed * speed_filter - ki_speed * car_speed_integeral;
    rotation_control_output = setting_turn_speed + kd_turn * kalmanfilter.Gyro_z;
    
  }

  //speed_control_output = 0;
  //rotation_control_output = 0;
  
  // combine balance and speed controller outputs to drive the motors
  pwm_left = balance_control_output - speed_control_output - rotation_control_output;
  pwm_right = balance_control_output - speed_control_output + rotation_control_output;

  Serial.print("PWM Left "); Serial.print(pwm_left);
  Serial.print(" PWM Right "); Serial.println(pwm_right);  
  
  // limit drive values in range -255 to 255
  pwm_left = constrain(pwm_left, -255, 255);
  pwm_right = constrain(pwm_right, -255, 255);
  
  if (motion_mode != STOP && (kalmanfilter_angle < balance_angle_min || balance_angle_max < kalmanfilter_angle)) {
    motion_mode = STOP;
    carStop();
  }

 // if (motion_mode == STOP) {
  if (!true) {    
    car_speed_integeral = 0;
    setting_car_speed = 0;
    pwm_left = 0;
    pwm_right = 0;
    carStop();
  } else {
    //Serial.println("Drive !!");
    if (pwm_left < 0) {
      digitalWrite(AIN1, 1);
      analogWrite(PWMA_LEFT, -pwm_left);
    } else {
      digitalWrite(AIN1, 0);
      analogWrite(PWMA_LEFT, pwm_left);
    }
    if (pwm_right < 0) {
      digitalWrite(BIN1, 1);
      analogWrite(PWMB_RIGHT, -pwm_right);
    } else {
      digitalWrite(BIN1, 0);
      analogWrite(PWMB_RIGHT, pwm_right);
    }
  }
}

// Interrupt service routine to get the encoder signals
void encoderCountRightA() {
  encoder_count_right_a++;
}

void encoderCountLeftA() {
  encoder_count_left_a++;
}

void carInitialize() {
  // Sets pins to outputs
  pinMode(AIN1, OUTPUT);
  pinMode(BIN1, OUTPUT);
  pinMode(PWMA_LEFT, OUTPUT);
  pinMode(PWMB_RIGHT, OUTPUT);
  pinMode(STBY_PIN, OUTPUT);
  carStop
  
  // Initialize the accelerometer
  Wire.begin();
  mpu.initialize();
  
  // Define functions to receive encoder signals as interrupts
  attachInterrupt(digitalPinToInterrupt(ENCODER_LEFT_A_PIN), encoderCountLeftA, CHANGE);
  attachPinChangeInterrupt(ENCODER_RIGHT_A_PIN, encoderCountRightA, CHANGE);
  
  // Set up a timer to call the balanceCar function in the background
  MsTimer2::set(5, balanceCar);
  MsTimer2::start();
  Serial.println("Timer Started");
}
